@@ -5,18 +5,16 @@
 
 use anyhow::Context;
 
-use buffered_reader::BufferedReader;
+use openpgp::parse::buffered_reader::{self, BufferedReader};
 
 use openpgp::cert::Cert;
-use openpgp::crypto::hash::Digest;
-use openpgp::parse::Parse;
+use openpgp::parse::{Cookie, Parse};
 use openpgp::types::HashAlgorithm::SHA1;
 use openpgp::Result;
 use sequoia_openpgp as openpgp;
 
 use std::convert::TryInto;
 use std::fmt::Display;
-use std::io::Read;
 
 /// GnuPG Keybox
 ///
@@ -27,6 +25,7 @@ use std::io::Read;
 /// records:
 ///
 /// ```rust
+/// # use sequoia_openpgp::parse::buffered_reader;
 /// # fn parse_keybox(reader: &mut dyn buffered_reader::BufferedReader<()>)
 /// #    -> sequoia_openpgp::Result<Vec<sequoia_openpgp::Cert>> {
 /// use sequoia_ipc::keybox::{Keybox, KeyboxRecord};
@@ -74,16 +73,58 @@ impl<'a> Keybox<'a> {
         let kbx_record = KeyboxRecord::new(offset, (&content[..len]).to_vec())?;
         Ok(kbx_record)
     }
-}
 
-impl<'a> Parse<'a, Keybox<'a>> for Keybox<'a> {
-    fn from_reader<R: 'a + Read + Send + Sync>(reader: R) -> Result<Self> {
-        let bio = buffered_reader::Generic::new(reader, None);
+    /// Reads from the given buffered reader.
+    ///
+    /// Implementations of this function should be short.  Ideally,
+    /// they should hand of the reader to a private function erasing
+    /// the readers type by invoking [`BufferedReader::into_boxed`].
+    pub fn from_buffered_reader<R>(reader: R) -> Result<Self>
+    where
+        R: BufferedReader<Cookie> + 'a,
+    {
         Ok(Keybox {
             offset: 0,
-            reader: Box::new(bio),
+            reader: buffered_reader::Adapter::new(reader).into_boxed(),
         })
     }
+
+    /// Reads from the given reader.
+    ///
+    /// The default implementation just uses
+    /// [`Parse::from_buffered_reader`], but implementations can
+    /// provide their own specialized version.
+    pub fn from_reader<R: 'a + std::io::Read + Send + Sync>(reader: R) -> Result<Self> {
+        Self::from_buffered_reader(
+            buffered_reader::Generic::with_cookie(reader,
+                                                  None,
+                                                  Default::default())
+                .into_boxed())
+    }
+
+    /// Reads from the given file.
+    ///
+    /// The default implementation just uses
+    /// [`Parse::from_buffered_reader`], but implementations can
+    /// provide their own specialized version.
+    pub fn from_file<P: AsRef<std::path::Path>>(path: P) -> Result<Self>
+    {
+        Self::from_buffered_reader(
+            buffered_reader::File::with_cookie(path.as_ref(),
+                                               Default::default())?
+                .into_boxed())
+    }
+
+    /// Reads from the given slice.
+    ///
+    /// The default implementation just uses
+    /// [`Parse::from_buffered_reader`], but implementations can
+    /// provide their own specialized version.
+    pub fn from_bytes<D: AsRef<[u8]> + ?Sized + Send + Sync>(data: &'a D) -> Result<Self> {
+        Self::from_buffered_reader(
+            buffered_reader::Memory::with_cookie(data.as_ref(), Default::default())
+                .into_boxed())
+    }
 }
 
 impl<'a> Iterator for Keybox<'a> {
@@ -385,7 +426,7 @@ impl OpenPGPRecordV1 {
     pub fn compute_checksum(&self) -> Result<Vec<u8>> {
         let hash_offset = self.data_offset() + self.data_length();
         let (hashed_data, _hash) = &self.bytes.split_at(hash_offset);
-        let mut ctx = SHA1.context()?;
+        let mut ctx = SHA1.context()?.for_digest();
         ctx.update(hashed_data);
         ctx.into_digest()
     }
