@@ -1,4 +1,5 @@
 use std::fmt;
+use std::io::Write;
 
 use sequoia_openpgp as openpgp;
 use openpgp::Error;
@@ -83,16 +84,17 @@ impl Keygrip {
     /// #   W2hrBY5x0sZ8H7JlAP47mCfCuRVBqyaePuzKbxLJeLe2BpDdc0n2izMVj8t9Cg==
     /// #   =QetZ
     /// #   -----END PGP PUBLIC KEY BLOCK-----".parse()?;
-    /// assert_eq!(Keygrip::of(cert.primary_key().mpis())?.to_string(),
+    /// assert_eq!(Keygrip::of(cert.primary_key().key().mpis())?.to_string(),
     ///            "DD143ABA8D1D7D09875D6209E01BCF020788FF77");
     /// # Ok(()) }
     /// ```
     pub fn of(key: &PublicKey) -> Result<Keygrip> {
         use openpgp::crypto::hash;
         use self::PublicKey::*;
-        let mut hash = HashAlgorithm::SHA1.context().unwrap();
+        let mut hash =
+            HashAlgorithm::SHA1.context().unwrap().for_digest();
 
-        fn hash_sexp_mpi(hash: &mut dyn hash::Digest, kind: char, mpi: &MPI)
+        fn hash_sexp_mpi(hash: &mut hash::Context, kind: char, mpi: &MPI)
         {
             // gcrypt's MPIs can be signed or unsigned.  Our MPIs are
             // always unsigned (well, for us they are opaque byte
@@ -115,7 +117,7 @@ impl Keygrip {
             hash_sexp(hash, kind, prefix, mpi.value());
         }
 
-        fn hash_sexp(hash: &mut dyn hash::Digest, kind: char, prefix: &[u8],
+        fn hash_sexp(hash: &mut hash::Context, kind: char, prefix: &[u8],
                      buf: &[u8])
         {
             write!(hash, "(1:{}{}:",
@@ -125,7 +127,7 @@ impl Keygrip {
             write!(hash, ")").unwrap();
         }
 
-        fn hash_ecc(hash: &mut dyn hash::Digest, curve: &Curve, q: &MPI)
+        fn hash_ecc(hash: &mut hash::Context, curve: &Curve, q: &MPI)
             -> Result<()>
         {
             for (i, name) in "pabgnhq".chars().enumerate() {
